#!/usr/bin/python3
# -*-coding:Utf-8 -*

import sys
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
import numpy as np
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import matplotlib.pyplot as plt


class CurseurEditableQt5(QWidget):
    """ensemble d'un curseur avec valeur éditable et label"""

    def __init__(self, numero, rang, valeur, phi, min=0.0, max=100.0, resol=1.0,
                 **kwargs):
        QWidget.__init__(self)

        self.n = numero
        self.val = valeur
        self.valphi = phi
        self.r = rang

        self.labelSlider = QLabel()
        self.labelSlider.setText('h n°')

        self.editRang = QLineEdit()
        self.editRang.setText(str(rang))
        self.editRang.setMaximumWidth(50)
        self.editRang.textChanged.connect(self.RangChange)

        self.CB = QCheckBox()
        self.CB.setChecked(True)
        self.CB.pressed.connect(self.CBchange)
        self.CBState = True  #  la propriété isChecked réagit en retard!

        self.slider = QSlider()
        self.MaxSlid = max
        self.slider.setOrientation(Qt.Vertical)
        self.slider.setMinimumHeight(50)
        self.slider.setTickPosition(QSlider.TicksAbove)
        self.slider.setMaximum(max)  # le slider est en entiers -> & correspond à 0.1ms
        self.slider.setSingleStep(resol)
        self.slider.setPageStep(10*resol)
        self.slider.setTickInterval(10*resol)
        self.slider.setValue(self.val)
        self.slider.valueChanged.connect(self.sliderChange)

        self.editSlider = QLineEdit()
        self.editSlider.setText(str(self.val))
        self.editSlider.setMaximumWidth(50)
        self.editSlider.textChanged.connect(self.lineChange)

        self.labelphi = QLabel()
        self.labelphi.setText('φ')

        self.editphi = QLineEdit()
        self.editphi.setText("0")
        self.editphi.setMaximumWidth(50)
        self.editphi.textChanged.connect(self.phiChange)

        layout = QGridLayout()
        layout.addWidget(self.labelSlider,0,0, Qt.AlignRight)
        layout.addWidget(self.editRang, 0, 1, Qt.AlignLeft)
        layout.addWidget(self.CB, 1, 0, Qt.AlignRight)
        layout.addWidget(self.slider, 1, 1, Qt.AlignLeft)
        layout.addWidget(self.editSlider, 2, 1, Qt.AlignLeft)
        layout.addWidget(self.labelphi, 3, 0, Qt.AlignRight)
        layout.addWidget(self.editphi, 3, 1, Qt.AlignLeft)

        self.setLayout(layout)

        #  initialisations
        self.flagEditS = False # sert à ne pas appeler 2 fois plot3graphes

    def RangChange(self):
        try:
            self.r = int(self.editRang.text())
            Window.plot3graphes(main, self.CBState, self.n, self.r, self.val, self.valphi)
        except ValueError:
            pass

    def sliderChange(self):
        if not self.flagEditS:
            self.val = self.slider.value()
            self.editSlider.setText(str(self.val))
        Window.plot3graphes(main, self.CBState, self.n, self.r, self.val, self.valphi)

    def lineChange(self):
        try:
            self.flagEditS = True
            self.val = float(self.editSlider.text())
            self.slider.setValue(self.val)
            self.flagEditS = False
        except ValueError:
            pass

    def phiChange(self):
        try:
            self.valphi = float(self.editphi.text())
            Window.plot3graphes(main, self.CBState, self.n, self.r, self.val, self.valphi)
        except ValueError:
            pass

    def setValues(self,rang, amp, phase):
        #  sert à imposer les valeurs par programmation pour les motifs pré-établis
        self.editRang.setText(str(rang))
        self.editRang.repaint()
        self.slider.setValue(int(amp))
        self.slider.repaint()
        self.editSlider.setText(str(amp))
        self.editSlider.repaint()
        self.editphi.setText(str(phase))
        self.editphi.repaint()

    def CBchange(self):
        self.CBState = not self.CBState
        Window.plot3graphes(main, self.CBState, self.n, self.r, self.val, self.valphi)


class Window(QWidget):
    def __init__(self, parent=None):
        super(Window, self).__init__(parent)

        self.setWindowTitle("Composition de signaux à partir du spectre")

        # barre de menus
        self.bar = QMenuBar()

        signal = self.bar.addMenu("Signal")

        sin = QAction("Sinusoïde", self)  # une ligne de menu avec raccourci.
        sin.setShortcut("Ctrl+s")
        signal.addAction(sin)  # Génère la str Sinusoïde

        carre = QAction("Carré", self)  # une ligne de menu avec raccourci.
        carre.setShortcut("Ctrl+c")
        signal.addAction(carre)

        tiers = QAction("2/3-1/3", self)  # une ligne de menu avec raccourci.
        tiers.setShortcut("Ctrl+2")
        signal.addAction(tiers)

        podium = QAction("Podium", self)  # une ligne de menu avec raccourci.
        podium.setShortcut("Ctrl+p")
        signal.addAction(podium)

        signal.triggered[QAction].connect(self.processtrigger)

        # a figure instance to plot on
        self.figure1 = plt.figure()
        self.figure1.patch.set_facecolor('white')

        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas1 = FigureCanvas(self.figure1)

        # create FFT axis
        self.ax1 = self.figure1.add_subplot(311)

        # create time axis for somme
        self.ax2 = self.figure1.add_subplot(312)

        # create time axis for 8 premiers harmoniques
        self.ax3 = self.figure1.add_subplot(313)

        #  variables liées aux sliders
        self.coches = [True]*7

        self.rangs = [2*i+1 for i in range(7)]

        self.amplitudes = [100.0 , 33.3, 20.0, 14.3, 11.10, 9.09, 7.69]  #  carré

        self.phases = [0.0]*7

        #  sliders
        self.curseurs = []
        for i in range(7):
            self.curseurs.append(CurseurEditableQt5(i, self.rangs[i], self.amplitudes[i], self.phases[i]))

        # set the layout______________________________________________________________________________________
        layout = QVBoxLayout()  # layout principal
        layoutFig = QVBoxLayout()  # contient le graphe et le slider
        layoutSlid = QHBoxLayout()  # contient les sliders des harmoniques
        self.layoutGraph = QVBoxLayout()  # contient le graphe et ses menus

        layoutSlid.addSpacing(100)
        for i in range(7):
            layoutSlid.addWidget(self.curseurs[i])
        layoutSlid.addSpacing(100)

        self.layoutGraph.addWidget(self.canvas1)

        ecran = QApplication.desktop()
        if QDesktopWidget.screenGeometry(ecran).height() > 800:
            # this is the Navigation widget
            # it takes the Canvas widget and a parent
            self.toolbar = NavigationToolbar(self.canvas1, self)
            self.layoutGraph.addWidget(self.toolbar)

        # layoutFig.addWidget(self.stackFig)
        layoutFig.addLayout(self.layoutGraph)  # insère self.layoutGraph  dans layoutFig
        layoutFig.addLayout(layoutSlid)  # insère layoutSlid  dans layoutFig

        layout.addWidget(self.bar)
        layout.addLayout(layoutFig)  # insère layoutFig dans le layout principal

        self.setLayout(layout)
        # ____________________________________________________________________________________________________

        self.Tt = []
        self.flagEditS = False
        self.plot3graphes(True, 0, 1, 100, 0.0)

    def processtrigger(self,q):
        self.fct = q.text()

        if self.fct=="Sinusoïde":
            for i in range(1, 7):
                self.rangs[i] = 2*i+1
                self.phases[i] = 0.0
                self.amplitudes[i] = 0.0
                self.curseurs[i].setValues(self.rangs[i], self.amplitudes[i], self.phases[i])
            self.rangs[0] = 1
            self.amplitudes[0] = 100.0
            self.phases[0] = 0.0
            self.curseurs[0].setValues(self.rangs[0], self.amplitudes[0], self.phases[0])

        if self.fct=="Carré":
            self.rangs = [2 * i + 1 for i in range(7)]
            self.amplitudes = [100.0, 33.3, 20.0, 14.3, 11.10, 9.09, 7.69]
            self.phases = [0.0] * 7
            for i in range(1, 7):
                self.curseurs[i].setValues(self.rangs[i], self.amplitudes[i], self.phases[i])

        if self.fct=="2/3-1/3":
            self.rangs = [2 * i + 1 for i in range(7)]
            self.amplitudes = [100.0, 0.0, 20.0, 14.3, 0.0, 9.09, 7.69]
            self.phases = [0.0, 0.0, -180, -180, 0.0, 0.0, 0.0]
            for i in range(1, 7):
                self.curseurs[i].setValues(self.rangs[i], self.amplitudes[i], self.phases[i])

        if self.fct=="Podium":
            self.rangs = [2 * i + 1 for i in range(7)]
            self.amplitudes = [100.0, 0.0, 20.0, 14.3, 0.0, 9.09, 7.69]
            self.phases = [0.0] * 7
            for i in range(1, 7):
                self.curseurs[i].setValues(self.rangs[i], self.amplitudes[i], self.phases[i])

    #  def paintEvent(self, a0: QWidget.paintEvent) -> None:
     #   print(QWidget.height(self))
     #   if QWidget.height(self) < 800:
     #       self.layoutGraph.removeWidget(self.toolbar)
     #       self.toolbar.draw()
     #   else:
     #       self.layoutGraph.addWidget(self.toolbar)
        

    def fonction(self, f, fe):
        resultat = [[], [], [], [], [], [], [], []]
        for i in range(1, 7): resultat[i] = []
        for t in self.Tt:
            somme = 0.0
            for i in range(0, 7):
                hi = self.amplitudes[i] / 100 * np.sin(2 * np.pi * self.rangs[i] * f * t + self.phases[i])  # harmonique de rang i
                somme = somme + hi
                resultat[i + 1].append(hi)
            resultat[0].append(somme)
        return resultat

    def plot3graphes(self, valide, n, rang, val, phi):
        self.coches[n] = valide
        self.rangs[n] = rang
        self.amplitudes[n] = val
        self.phases[n] = phi * np.pi /180.0
        Te = 0.0001
        self.Tt = np.arange(400) * Te
        nulle = [0]*400

        liste_s = self.fonction(50.0, 10000.0)

        #  création du spectre
        freq = np.arange(0, 800, 1)
        spectre = [0.0]*800
        for i in range(7): spectre[50*self.rangs[i]] = self.amplitudes[i]

        self.ax3.set_xlabel("temps")
        # discards the old graph
        self.ax1.clear()
        self.ax2.clear()
        self.ax3.clear()

        # plot data
        self.ax3.plot(freq, spectre)

        self.ax1.plot(self.Tt, liste_s[0], 'r')
        self.ax1.plot(self.Tt, liste_s[1])
        self.ax1.plot(self.Tt, nulle, 'black')

        for i in range(7):
            if self.coches[i]: self.ax2.plot(self.Tt, liste_s[i + 1])

        self.ax3.set_xlabel("fréquence")
        self.ax3.set_ylabel("amplitude")
        self.ax1.set_xlabel("temps")
        self.ax2.set_xlabel("temps")
        #  self.ax3.set_ylim([-1.1, 1.1])

        self.ax1.grid(True)
        self.ax2.grid(True)
        self.ax3.grid(True)

        # refresh canvas
        self.canvas1.draw()


if __name__ == '__main__':
    app = QApplication(sys.argv)

    main = Window()
    main.show()
    sys.exit(app.exec_())
